package lightcontainer.domains.server;

import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.interfaces.MulticastSPR;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.reader.HelloRule;

import java.io.IOException;
import java.net.*;

/**
 * StoreMulticastRunnable
 * <p>
 * Class listening to the announcement of new StoreBackEnd.
 * Allowing it to be used as a storage unit.
 *
 * @author Jérémi NIHART <j.nihart@student.helmo.be>
 * @version 1.0
 * @see Runnable
 * @since 1.0
 */
public class MulticastServerListener implements Runnable {
    // Variable
    private final String multicast_address;
    private final int multicast_port;
    private final MulticastSPR repository;
    private final ProtocolRepository protocolRep;

    private final byte[] buffer = new byte[256];
    private MulticastSocket listener;

    // Constructor
    public MulticastServerListener(MulticastSPR repository, ProtocolRepository protocolRep, String multicast_address, int multicast_port) {
        this.repository = repository;
        this.protocolRep = protocolRep;
        this.multicast_address = multicast_address;
        this.multicast_port = multicast_port;
        repository.setServerListener(this);
    }

    /**
     * Start Multicast listening on indicated port and IP group.
     *
     * @see MulticastSocket#receive(DatagramPacket)
     * @see DatagramPacket
     * @since 1.0
     */
    @Override
    public void run() {
        try {
            // Create a new listening socket
            this.listener = new MulticastSocket(this.multicast_port);
            // Create an identifier for the multicast group on the specified ip
            InetAddress listener_group = InetAddress.getByName(this.multicast_address);
            // Creation of a packet for the information received
            DatagramPacket packet = new DatagramPacket(this.buffer, this.buffer.length);
            // Add the listener to the multicast group
            this.listener.joinGroup(listener_group);
            while (true) {
                // Read the packet received and build a string of characters
                this.listener.receive(packet);
                String data = new String(packet.getData(), 0, packet.getLength());
                // Create a new StoreBacked (try used in the case of an error to maintain the listening loop)
                try {
                    // TODO Récupérer le port du message du packet et le setup (add description of the line).
                    HelloRule.Result readerResult = protocolRep.executeReader(data);
                    System.out.printf("Réception en multicast : Domain=%s | Port=%d\n", readerResult.getDomain(), readerResult.getPort());

                    Socket socket = new Socket(packet.getAddress(), readerResult.getPort());
                    // Create the store processor
                    StoreProcessor storeProcessor = new StoreProcessor(socket, readerResult.getDomain(), null); // TODO <!!!> : Voir comment on procède get via repo ou ici ?!
                    // Add the store processor to its repository
                    this.repository.addStore(storeProcessor);
                } catch (IOException ignore) {
                    ignore.printStackTrace();
                }
            }
        } catch (Exception ignore) {
        }
    }

    /**
     * Closes the MulticastSocket connection and aborts the listening and infinite listening loop.
     *
     * @see MulticastServerListener#run()
     * @since 1.0
     */
    public void stop() {
        this.listener.close();
    }
}
