package lightcontainer.protocol;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class ProtocolReader {
    // Variables
    private final Pattern rulePattern;

    // Constructor
    protected ProtocolReader(String pattern) {
        this.rulePattern = Pattern.compile(pattern);
    }

    public enum ResultCmdReceiver {
        CLIENT,
        STOREBACKEND
    }

    public enum FileOptions {

        /**
         * Ne lire/ecrire aucun fichier
         */
        NONE,

        /**
         * Lire un fichier : Implementer la méthode 'read'
         */
        READ,

        /**
         * Ecrire un fichier : Implementer la méthode 'write'
         */
        WRITE
    }

    /**
     * Modèle utilisé par tout les résultats des règles de protocol.
     * Lorsqu'il retourne son résultat, on vérifie si il y a une demande de lecture/écriture de fichier depuis le réseau. Si oui on appel ces méthodes, sinon on ne fait rien.
     * Ensuite on regarde après l'exécution de ces méthode ou non si il y a une commande de retour ou non et l'envoyons au receiver spécifié par la commande.
     */
    public abstract class ProtocolResult {

        /**
         * Permet de déterminer si la class souhaite lire ou écrire un fichier.
         */
        private final FileOptions option;

        /**
         * Command qui sera renvoyée par exemple au client
         */
        private String resultCommand;

        /**
         * Désigne vers ou cette commande est envoyée.
         * ResultCmdReceiver.CLIENT : Signifie que cette commande va être directement revoyée au client.
         * ResultCmdReceiver.STOREBACKEND : Signifie que cette commande va être envoyée dans la file de tâche du server et êre en attente d'envoie à un StorBackEnd
         */
        private ResultCmdReceiver receiver;


        /**
         * Spécifier une option correct afin de permettre au système de détecter ce qu'il doit exécuter.
         * @param option Option
         */
        public ProtocolResult(FileOptions option) {
            this.option = option;
        }


        public ResultCmdReceiver getReceiver() {
            return receiver;
        }


        /**
         * Récupérer la commande à envoyer
         * @return
         */
        public String getResultCommand() {
            return resultCommand;
        }

        /**
         * Mettre la commande à envoyer
         * @param resultCommand Commande à envoyer
         * @param receiver Le receveur de cette commande
         */
        public void setResultCommand(String resultCommand, ResultCmdReceiver receiver) {
            this.resultCommand = resultCommand;
            this.receiver = receiver;
        }

        /**
         * Permet de récupérer l'option associée au protocol.
         * @return L'option associée
         */
        public FileOptions getOption() {
            return option;
        }

        /**
         * Permet de lire un fichier. Cad reçevoir le contenu d'un fichier provenant du réseau.
         * Redéfinissez cette méthode pour l'utiliser
         * @param reader Buffer rempli du fichier
         */
        public void read(BufferedReader reader) {}


        /**
         * Permet d'écrire un fichier. Cad reçevoir le contenu d'un fichier provenant du réseau
         * @param writer Buffer à remplir, et qui sera envoyer au destinataire
         */
        public void write(BufferedWriter writer) {}

    }

    /**
     * Permet de lancer la décomposition d'une commande pour en extraire les données
     * @param data Contenu de la commande
     */
    public <T extends ProtocolResult> T execute(String data) {
        Matcher ruleMatcher = this.rulePattern.matcher(data);

        if (ruleMatcher.matches()) {
            String[] groups = new String[ruleMatcher.groupCount()];

            for (int i = 1; i <= groups.length; ++i)
                groups[i - 1] = ruleMatcher.group(i);

            return onExecuted(groups);
        }
        return null;
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     * @param data Paramètres pour créer la commande.
     */
    protected abstract <T> T onExecuted(String... data);
}
